@@ -1,7 +1,7 @@
 import os
 import argparse
 from flask import current_app
-from flask.ext.script import Manager
+from flask_script import Manager
 from alembic import __version__ as __alembic_version__
 from alembic.config import Config as AlembicConfig
 from alembic import command
@@ -35,15 +35,20 @@ class Config(AlembicConfig):
 class Migrate(object):
     def __init__(self, app=None, db=None, directory='migrations', **kwargs):
         self.configure_callbacks = []
+        self.db = db
         self.directory = directory
+        self.alembic_ctx_kwargs = kwargs
         if app is not None and db is not None:
-            self.init_app(app, db, directory, **kwargs)
+            self.init_app(app, db, directory)
 
-    def init_app(self, app, db, directory=None, **kwargs):
+    def init_app(self, app, db=None, directory=None, **kwargs):
+        self.db = db or self.db
         self.directory = directory or self.directory
+        self.alembic_ctx_kwargs.update(kwargs)
         if not hasattr(app, 'extensions'):
             app.extensions = {}
-        app.extensions['migrate'] = _MigrateConfig(self, db, **kwargs)
+        app.extensions['migrate'] = _MigrateConfig(self, self.db,
+            **self.alembic_ctx_kwargs)
 
     def configure(self, f):
         self.configure_callbacks.append(f)
@@ -63,11 +68,16 @@ class Migrate(object):
             config.cmd_opts = argparse.Namespace()
         for opt in opts or []:
             setattr(config.cmd_opts, opt, True)
-        if x_arg is not None:
-            if not getattr(config.cmd_opts, 'x', None):
-                setattr(config.cmd_opts, 'x', [x_arg])
+        if not hasattr(config.cmd_opts, 'x'):
+            if x_arg is not None:
+                setattr(config.cmd_opts, 'x', [])
+                if isinstance(x_arg, list) or isinstance(x_arg, tuple):
+                    for x in x_arg:
+                        config.cmd_opts.x.append(x)
+                else:
+                    config.cmd_opts.x.append(x_arg)
             else:
-                config.cmd_opts.x.append(x_arg)
+                setattr(config.cmd_opts, 'x', None)
         return self.call_configure_callbacks(config)
 
 
@@ -82,7 +92,7 @@ MigrateCommand = Manager(usage='Perform database migrations')
                        help=("Multiple databases migraton (default is "
                              "False)"))
 def init(directory=None, multidb=False):
-    """Generates a new migration"""
+    """Creates a new migration repository"""
     if directory is None:
         directory = current_app.extensions['migrate'].directory
     config = Config()
@@ -117,10 +127,11 @@ def init(directory=None, multidb=False):
                              "output instead"))
 @MigrateCommand.option('--autogenerate', dest='autogenerate',
                        action='store_true', default=False,
-                       help=('Populate revision script with andidate migration '
-                             'operatons, based on comparison of database to '
-                             'model'))
-@MigrateCommand.option('-m', '--message', dest='message', default=None)
+                       help=('Populate revision script with candidate '
+                             'migration operations, based on comparison of '
+                             'database to model'))
+@MigrateCommand.option('-m', '--message', dest='message', default=None,
+                       help='Revision message')
 @MigrateCommand.option('-d', '--directory', dest='directory', default=None,
                        help=("migration script directory (default is "
                              "'migrations')"))
@@ -160,11 +171,14 @@ def revision(directory=None, message=None, autogenerate=False, sql=False,
 @MigrateCommand.option('-d', '--directory', dest='directory', default=None,
                        help=("migration script directory (default is "
                              "'migrations')"))
+@MigrateCommand.option('-x', '--x-arg', dest='x_arg', default=None,
+                       action='append', help=("Additional arguments consumed "
+                                              "by custom env.py scripts"))
 def migrate(directory=None, message=None, sql=False, head='head', splice=False,
-            branch_label=None, version_path=None, rev_id=None):
+            branch_label=None, version_path=None, rev_id=None, x_arg=None):
     """Alias for 'revision --autogenerate'"""
     config = current_app.extensions['migrate'].migrate.get_config(
-        directory, opts=['autogenerate'])
+        directory, opts=['autogenerate'], x_arg=x_arg)
     if alembic_version >= (0, 7, 0):
         command.revision(config, message, autogenerate=True, sql=sql,
                          head=head, splice=splice, branch_label=branch_label,
@@ -178,7 +192,7 @@ def migrate(directory=None, message=None, sql=False, head='head', splice=False,
 @MigrateCommand.option('-d', '--directory', dest='directory', default=None,
                        help=("migration script directory (default is "
                              "'migrations')"))
-def edit(revision='current', directory=None):
+def edit(directory=None, revision='current'):
     """Edit current revision."""
     if alembic_version >= (0, 8, 0):
         config = current_app.extensions['migrate'].migrate.get_config(
@@ -224,8 +238,8 @@ def merge(directory=None, revisions='', message=None, branch_label=None,
                        help=("migration script directory (default is "
                              "'migrations')"))
 @MigrateCommand.option('-x', '--x-arg', dest='x_arg', default=None,
-                       help=("Additional arguments consumed by "
-                             "custom env.py scripts"))
+                       action='append', help=("Additional arguments consumed "
+                                              "by custom env.py scripts"))
 def upgrade(directory=None, revision='head', sql=False, tag=None, x_arg=None):
     """Upgrade to a later version"""
     config = current_app.extensions['migrate'].migrate.get_config(directory,
@@ -245,8 +259,8 @@ def upgrade(directory=None, revision='head', sql=False, tag=None, x_arg=None):
                        help=("migration script directory (default is "
                              "'migrations')"))
 @MigrateCommand.option('-x', '--x-arg', dest='x_arg', default=None,
-                       help=("Additional arguments consumed by "
-                             "custom env.py scripts"))
+                       action='append', help=("Additional arguments consumed "
+                                              "by custom env.py scripts"))
 def downgrade(directory=None, revision='-1', sql=False, tag=None, x_arg=None):
     """Revert to a previous version"""
     config = current_app.extensions['migrate'].migrate.get_config(directory,
